@@ -1,11 +1,9 @@
-import buble from 'rollup-plugin-buble';
-import resolve from 'rollup-plugin-node-resolve';
-import replace from 'rollup-plugin-replace';
+import nodeResolve from '@rollup/plugin-node-resolve';
+import replace from '@rollup/plugin-replace';
 
 const plugins = [
-	buble({ exclude: 'node_modules/**' }),
-	resolve(),
-	replace({ DEBUG: false })
+	nodeResolve(),
+	replace({ DEBUG: false, preventAssignment: true })
 ];
 
 export default [
@@ -15,7 +13,7 @@ export default [
 		external: ['sourcemap-codec'],
 		plugins,
 		output: {
-			file: 'dist/magic-string.es.js',
+			file: 'dist/magic-string.es.mjs',
 			format: 'es',
 			exports: 'named',
 			sourcemap: true
